﻿# -*- coding: utf-8 -*-
"""
テンプレート言語 XML を読み取って、テンプレートを翻訳します。
スクリプトと同じディレクトリの、TemplateTranslationDefs[<言語名>].xml
ファイルをロードします。

MakeTranslateList で、.NET クラスを使用しています。
"""

import sys
import clr
import cse
import re
clr.AddReference("System.Xml")
import System.Xml.XmlDocument
import System.IO.Path as path
import System.IO.File as file

# 言語情報ディクショナリ
langlist = None
sheetlist = None

# 正規表現オブジェクト
rx = re.compile(r"(\.[tc]\.\w+)\.(.+)$", re.I | re.L)
valrx = re.compile(r"^value\[(.+)\]$", re.I | re.L)

# シート情報格納クラス
class SheetVersionInfo:
  "シート情報格納クラス"
  def __init__(self, Name, Title, MjVer, MnVer):
    self.Name = Name
    self.Title = Title
    self.MjVer = MjVer
    self.MnVer = MnVer


# 言語XML読み込みメソッド
def MakeTranslateList(xmlPath):
  global langlist
  global sheetlist
  
  # XML 言語コードを読み込む
  outDic = {}	# csePath をキー、「翻訳キー＆翻訳文 の辞書」を値にした、辞書
  sheetDic = {}	# シート名をキー、シートタイトル等を値にした、辞書
  
  xml = System.Xml.XmlDocument()
  xml.Load(xmlPath)
  
  root = xml.SelectSingleNode("/cseTemplateLanguageDefinitions")
  langname = root.Attributes.GetNamedItem("language").Value
  
  for sheetnode in root.SelectNodes("sheet"):
    # sheet ノードの属性を取得
    try:
      sheetName = sheetnode.Attributes.GetNamedItem("name").Value
      sheetTitle = sheetnode.Attributes.GetNamedItem("title").Value
      sheetMjVer = sheetnode.Attributes.GetNamedItem("majorVer").Value
      sheetMnVer = sheetnode.Attributes.GetNamedItem("minorVer").Value
    except AttributeError, e:
      sys.stderr.write("'sheet' node doesn't have enough attributes.\n")
      continue
    
    # シート情報オブジェクトを作成
    try:
      sheetMjVerNum = int(sheetMjVer)
      sheetMnVerNum = int(sheetMnVer)
    except ValueError, e:
      sys.stderr.write(str(e) + "\n")
      continue 
    sheetDic[sheetName] = SheetVersionInfo(sheetName, sheetTitle, sheetMjVerNum, sheetMnVerNum)
    
    for itemnode in sheetnode.SelectNodes("lngitem"):
      try:
        localePath = itemnode.Attributes.GetNamedItem("key").Value
      except AttributeError, e:
        sys.stderr.write("'lngitem' node has no attribute 'key'.\n")
        continue
      
      Value = itemnode.InnerText
      
      # CSEパス部分を切り出す
      m = rx.match(localePath)
      Path = sheetName + m.group(1)
      ItemPath = m.group(2)
      
      # Path の辞書がなければ作成
      if Path not in outDic:
        outDic[Path] = {}
      
      # 追加
      outDic[Path][ItemPath] = Value
      
    
  langlist = outDic
  sheetlist = sheetDic


# 言語情報ファイルの読取
def LoadTranslationFile():
  global langlist
  
  xmlname = "TemplateTranslationDefs[%s].xml" % cse.appsetting.CurrentLngName
  
  # フォルダ名を取得
  xmlpath = path.Combine(path.GetDirectoryName(__file__), xmlname)
  
  # ファイルディクショナリの読取
  if file.Exists(xmlpath):
    MakeTranslateList(xmlpath)



# 出力シートチェックメソッド
def TranslateTemplates(s, e):
  global langlist
  global sheetlist
  if langlist == None or sheetlist == None:
    return
  
  # プラグイン実行イベントデータ
  sheetTabItems = e.Value
  
  for sti in sheetTabItems:
    # バージョンを比較して、異なる場合は翻訳しない。
    sheetValue = sti.Sheet
    if sheetValue.Name not in sheetlist:
      sys.stderr.write("'" + sheetValue.Name + "' is not defined.\n")
      continue
    svi = sheetlist[sheetValue.Name]
    if sheetValue.MajorVer != svi.MjVer or sheetValue.MinorVer != svi.MnVer:
      sys.stderr.write("'%s' ver. %d.%d is not defined.\n" % (sheetValue.Name, sheetValue.MajorVer, sheetValue.MinorVer))
      continue
    
    # タイトルの反映
    sheetValue.Title = svi.Title
    
    # 親側、子側定義コンテンツでループ
    for contset in cse.getcontssets(sheetValue, True):
      for cont in contset:
        # Cseパス文字列を取得
        path = cse.deparsepath(cont)
        
        if path in langlist:
          # 言語ディクショナリに、Cseパスのキーが存在したら、
          # その 翻訳キー＆翻訳文 ごとにテキストの置き換えを行う
          for k, v in langlist[path].items():
            if k.lower() == "title":
              cont.Title = v
            elif k.lower() == "note":
              cont.Note = v
            elif k.lower() == "value":
              # index なし value
              if str(cont.type) == "number":
                # 数値
                try:
                  cont.Value = int(v)
                except:
                  pass
                
              elif str(cont.type) == "text" or str(cont.type) == "multitext":
                # テキスト
                cont.Value = v
                
            elif valrx.match(k) != None:
              # index 付き value
              index = valrx.match(k).group(1)
              
              if str(cont.type) == "list":
                # リスト
                try:
                  index = int(index)
                except:
                  continue
                if 0 <= index < len(cont.Value):
                  cont.Value[index] = v
                
              elif str(cont.type) == "check" or str(cont.type) == "radio":
                # チェックorラジオ
                for chk in cont.Value:
                  if chk.Id == index:
                    chk.Text = v
                
    


# XML 読取の実行と、イベントハンドラの登録
LoadTranslationFile()
cse.addhandler("TemplateLoaded", TranslateTemplates)

